<?php
$version = '0.1.0-beta';

// Source: https://www.php.net/manual/fr/function.copy.php#91010
function recurse_copy($src,$dst) {
    $dir = opendir($src);
    @mkdir($dst);
    while(false !== ( $file = readdir($dir)) ) {
        if (( $file != '.' ) && ( $file != '..' )) {
            if ( is_dir($src . '/' . $file) ) {
                recurse_copy($src . '/' . $file,$dst . '/' . $file);
            }
            else {
                copy($src . '/' . $file,$dst . '/' . $file);
            }
        }
    }
    closedir($dir);
}

// Default config.json and user.json file contents
$config_json = '{'.PHP_EOL.'"source": "./source/",'.PHP_EOL.'"destination": "D:/travail-ecrit/"'.PHP_EOL.'}';
$user_json = '{'.PHP_EOL.'"prenom": "Prénom",'.PHP_EOL.'"nom": "Nom",'.PHP_EOL.'"classe": "nINF-XX-XX",'.PHP_EOL.'"machine": "LMB-XXX-XX"'.PHP_EOL.'}';

// display header
echo 'MultiSync v'.$version.PHP_EOL;
echo '*********************'.PHP_EOL;
echo PHP_EOL;

// generate config.json if it doesn't exist
if (!file_exists('config.json')) {
    file_put_contents('config.json', $config_json);
    echo 'Fichier config.json non trouvé, génération de celui-ci. Remplissez-le et relancez le script.'.PHP_EOL;
    exit;
}

// Read config.json
$config = json_decode(file_get_contents('config.json'));
if (!$config) {
    die('Vérifiez votre fichier config.json!'. PHP_EOL);
}

// Clean data from config.json
$config->source = str_replace('\\', '/', $config->source);
if (substr($config->source, -1) != '/') {
    $config->source .= '/';
}
if (in_array($config->source, ['bin/', 'rendu/', './bin/', './rendu/'])) {
    die('Erreur: config.json contient une valeur invalide pour "source"'.PHP_EOL);
}

$config->destination = str_replace('\\', '/', $config->destination);
if (substr($config->destination, -1) != '/') {
    $config->destination .= '/';
}
if (in_array($config->destination, ['bin/', 'rendu/', './bin/', './rendu/'])) {
    die('Erreur: config.json contient une valeur invalide pour "destination"'.PHP_EOL);
}

// Create destination directory if not exist
if (!file_exists($config->destination)) {
    mkdir($config->destination, 0777, true);
    echo 'Création du répertoire de travail'.PHP_EOL;
}

// Create user.json file if not exist
if (!file_exists($config->destination.'user.json')) {
    file_put_contents($config->destination.'user.json', $user_json);
    echo 'Création du fichier user.json'.PHP_EOL;
    exit;
}

// Find the last part already present in the destination
$partie = 0;
while (is_dir($config->destination.'partie-'.($partie + 1))) {
    $partie++;
}

// If found, fetch the directory
if ($partie > 0) {
    // Get user info from user.json
    $user = json_decode(file_get_contents($config->destination.'user.json'));
    if (!$user) {
        die('Vérifiez votre fichier user.json!'. PHP_EOL);
    }
    // Create folder in the "rendu" directory
    $root = 'rendu/'.$user->classe.'/'.$user->nom.'_'.$user->prenom.'_'.$user->machine.'/';
    @mkdir($root, 0777, true);
    // If already present, everything has already been fetched
    if (is_dir($root.'partie-'.$partie)) {
        die('Rien à faire, '.$user->prenom.' '.$user->nom.' a terminé son travail.'.PHP_EOL);
    }
    // Else, copy
    recurse_copy($config->destination.'partie-'.$partie, $root.'partie-'.$partie);
    echo 'Récupération de la partie '.$partie.' de '.$user->prenom.' '.$user->nom.' vers '.$root.PHP_EOL;
}

// If a following part is found in the source, send it to the destination
if (is_dir($config->source.'partie-'.($partie + 1))) {
    recurse_copy($config->source.'partie-'.($partie + 1), $config->destination.'partie-'.($partie + 1));
    echo 'Distribution de la partie '.($partie + 1).PHP_EOL;
}
